#include <stdint.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <avr/eeprom.h>
#include "hardware.h"

static void sendInt(uint16_t value)
{
	// show number
	uint16_t div = 10000;
	uint8_t firstChar = 1;
	uint8_t count = 0;
	while (div) {
		uint8_t digit = value / div;
		value -= digit * div;
		if (firstChar) {
			if (digit || (div == 1)) {
				writeChar(digit + '0');
				firstChar = 0;
				count++;
			}
		} else {
			writeChar(digit + '0');
			count++;
		}
		div /= 10;
	}
	
	// fill until line end with spaces
	while (count < 3) {
		writeChar(32);
		count++;
	}
}

static void writeColor(uint32_t color)
{
	uint8_t r = (color >> 16) & 0xff;
	uint8_t g = (color >> 8) & 0xff;
	uint8_t b = color & 0xff;
	r >>= 3;
	g >>= 2;
	b >>= 3;
	writeChar((r << 3) | (g >> 3));
	writeChar(((g & 7) << 5) | b);
}

static void drawStringPrefix(uint8_t x, uint8_t y, uint32_t color, uint8_t size)
{
	writeChar(0x53);
	writeChar(x);
	writeChar(y);
	writeChar(2);  // 8x12 font
	writeColor(color);
	writeChar(size);  // width
	writeChar(size);  // height
}

void drawString(uint8_t x, uint8_t y, uint32_t color, uint8_t size, const char* string)
{
	drawStringPrefix(x, y, color, size);
	while (1) {
		writeChar(*string);
		if (!(*string)) break;
		string++;
	}
	readChar();
}

void drawInt(uint8_t x, uint8_t y, uint32_t color, uint8_t size, uint16_t value)
{
	drawStringPrefix(x, y, color, size);
	sendInt(value);
	writeChar(0);  // terminator
	readChar();
}

void drawTime(uint8_t x, uint8_t y, uint32_t color, uint8_t hour, uint8_t minute, uint8_t second)
{
	drawStringPrefix(x, y, color, 1);
	writeChar((hour / 10) + '0');
	writeChar((hour % 10) + '0');
	writeChar(':');
	writeChar((minute / 10) + '0');
	writeChar((minute % 10) + '0');
	writeChar(':');
	writeChar((second / 10) + '0');
	writeChar((second % 10) + '0');
	writeChar(0);  // terminator
	readChar();
}

void drawRectangle(uint8_t x1, uint8_t y1, uint8_t x2, uint8_t y2, uint32_t color)
{
	writeChar(0x72);
	writeChar(x1);
	writeChar(y1);
	writeChar(x2);
	writeChar(y2);
	writeColor(color);
	readChar();
}

void initDisplay(void)
{
	// wait until demo of the display is finished
	delayMs(4000);
	delayMs(4000);
	delayMs(4000);

	// send OLED auto baud character
	writeChar(0x55);
	delayMs(100);
	clearReceiveBuffer();
	delayMs(100);

	// clear screen
	writeChar(0x45);
	readChar();
	
	// set opaque-mode for text
	writeChar(0x4f);
	writeChar(1);
	readChar();
	
	// pen size 0 for solid-mode
	writeChar(0x70);
	writeChar(0);
	readChar();
}
